%----------------------------------------------------------------------
%the sparse coding problem with two l1 regularization entries, one
%is the l1-norm of coding coef, the other is the l1-norm of coding
%coef minus the mean
%the object funciton is
% arg min{||y-Ax||_F^2+tau1*||x||_1+tau2*||x-m||_1}
% using surrogate function with iterative algorithm

%           Inputs
%                 ipts           -         an input data structure
%                     .D         -         the A matrix, each column vector
%                                          of A denotes a training sample
%                     .tau1      -         the parameter tau1
%                     .tau2      -         the paremeter tau2
%                     .m         -         the mean of coeficient
%                     .y         -         an input sample column vector
%                     .initM     -         initial method
%                     .twist     -         true: twist acceleration;false:no
%                                          accleration
%                     .citeT    -         the relative citeration threhold,
%                                          default:1e-4
%                     .cT        -         the stop citeration of c, the
%                                          threhold
%
%                 par            -         an input parameter structure
%                     .nIter     -         the total iteration number
%                     .cRatio    -         the times of c to max eigenvalue
% 
%            Outputs
%                opts            -         an output datastructure
%                     .x         -         the output spase coef
%                     .ert       -         the total error in iteration
%----------------------------------------------------------------------

function [opts] = doulbe_l1 ( ipts , par)

switch lower(ipts.initM)
    case {'zero'}
        x(:,1)  =  zeros(size(ipts.D,2),1);
    case {'transpose'}
        x(:,1)  =  ipts.D'*ipts.y;
    case {'pinv'}
        x(:,1)  =  pinv(ipts.D)*ipts.y;
    otherwise
        error('Nonknown method!');
end

A         =         ipts.D;
tau1      =         ipts.tau1;
tau2      =         ipts.tau2;
twist     =         ipts.twist;
mu        =         ipts.m;
y         =         ipts.y;
nIter     =         par.nIter;
if ~isfield(par,{'eigenv'})
c         =         par.cRatio*find_max_eigenv(A'*A);
else
c         =         par.cRatio*par.eigenv ;
end

for_ever           =         1;
IST_iters          =         0;
TwIST_iters        =         0;
sparse_sign        =         1;
verbose            =         1;
enforceMonotone    =         1;
lam1               =         1e-4;   %default eigenvalues
lamN               =         1;      %default eigenvalues
rho0               =         (1-lam1/lamN)/(1+lam1/lamN); 
alpha              =         2/(1+sqrt(1-rho0^2));        %default,user can set
beta               =         alpha*2/(lam1+lamN);         %default,user can set

%-------------------------------------------------------------
%start of main loop
%-------------------------------------------------------------
if ~twist
for n_it = 2 : nIter
    
    ert(n_it-1)      =       norm(y-A*x(:,n_it-1),2)^2+...
        tau1*norm(x(:,n_it-1),1)+tau2*norm(x(:,n_it-1)-mu,1);
    fprintf('Iteration:%f  Total gap:%f\n',n_it,ert(n_it-1));
%     v_temp           =       1/c*(y-A*x(:,n_it-1));
%     v_temp2          =       [A(:,1:size(A,1))'*v_temp;v_temp];
%     v                =       v_temp2 + x(:,n_it-1);
    v        =       1/c*A'*(y-A*x(:,n_it-1))+x(:,n_it-1);
    x(:,n_it)        =       NL_Shrinkage ( v, tau1/c, tau2/c, mu );    
end

else
    
   xm2       =      x(:,1);
   xm1       =      x(:,1);
   prev_f    =      norm(y-A*x(:,1),2)^2+...
                    tau1*norm(x(:,1),1)+tau2*norm(x(:,1)-mu,1);  
   for n_it = 2 : nIter;

   ert(n_it-1)      =       norm(y-A*x(:,n_it-1),2)^2+...
        tau1*norm(x(:,n_it-1),1)+tau2*norm(x(:,n_it-1)-mu,1);
%    fprintf('Iteration:%f  Total gap:%f\n',n_it,ert(n_it-1));
    
    while for_ever
        % IST estimate
%            Dim1_A           =       size(A,1);           
%            v_temp           =       1/c*(y-A*xm1);
%            v_temp2          =       [A(:,1:Dim1_A)'*v_temp;v_temp];
%            v                =       v_temp2 + xm1;

       v        =       1/c*A'*(y-A*xm1)+xm1;
       x_temp           =       NL_Shrinkage ( v, tau1/c, tau2/c, mu );
    
        if (IST_iters >= 2) | ( TwIST_iters ~= 0)
            % set to zero the past when the present is zero
            % suitable for sparse inducing priors
            if sparse_sign
                mask    =   (x_temp ~= 0);
                xm1     =   xm1.* mask;
                xm2     =   xm2.* mask;
            end
            % two-step iteration
            xm2    =   (alpha-beta)*xm1 + (1-alpha)*xm2 + beta*x_temp;
            % compute residual
            f      =   norm(y-A*xm2,2)^2+...
                       tau1*norm(xm2,1)+tau2*norm(xm2-mu,1);
            if (f > prev_f) & (enforceMonotone)
                TwIST_iters   =  0;  % do a IST iteration if monotonocity fails
            else
                TwIST_iters =   TwIST_iters+1; % TwIST iterations
                IST_iters   =    0;
                x_temp      =   xm2;
                if mod(TwIST_iters,10000) ==0
                   c = 0.9*c; 
                end
                break;  % break loop while
            end
        else
            f     =     norm(y-A*x_temp,2)^2+...
                        tau1*norm(x_temp,1)+tau2*norm(x_temp-mu,1);
            if f > prev_f
                % if monotonicity  fails here  is  because
                % max eig (A'A) > 1. Thus, we increase our guess
                % of max_svs
                c         =    2*c;        
                if verbose
%                     fprintf('Incrementing c=%2.2e\n',c);
                end
                if  c > ipts.cT
                    break;  % break loop while    
                end
                IST_iters = 0;
                TwIST_iters = 0;
            else
                TwIST_iters = TwIST_iters + 1;
                break;  % break loop while
            end
        end
    end

    citerion      =   abs(f-prev_f)/prev_f;
    if citerion < ipts.citeT | c > ipts.cT
%        fprintf('Stop!\n c=%2.2e\n citerion=%2.2e\n',c,citerion);
       break;
    end
    
    xm2           =   xm1;
    xm1           =   x_temp;
    x(:,n_it)   =   x_temp;
    prev_f        =   f;
    
   end

end
%--------------------------------------------------------------------------
% end of main loop
%--------------------------------------------------------------------------
opts.x     =       x(:,end);
opts.ert   =       ert;